﻿using System;
using System.Collections.Generic;
using System.Text;

namespace ELTE.Basics.GeometricShapes
{
    /// <summary>
    /// Vízszintes vonal típusa.
    /// </summary>
    class HorisontalLine : Shape
    {
        /// <summary>
        /// Alakzat kerületének lekérdezése.
        /// </summary>
        public override Double Perimeter
        {
            get { return 2 * _Radious; }
        }

        /// <summary>
        /// Vízszintes vonal létrehozása.
        /// </summary>
        /// <param name="center">Középpont.</param>
        /// <param name="length">Hossz.</param>
        public HorisontalLine(Point center, Double length)
            : base(center, length / 2)
        { }

        /// <summary>
        /// Vízszintes vonal létrehozása.
        /// </summary>
        /// <param name="left">Bal végpont.</param>
        /// <param name="right">Jobb végpont.</param>
        public HorisontalLine(Point left, Point right)
        {
            if (left.X == right.X && left.Y == right.Y)
                throw new BadParameterException("A bal és jobb pontok azonosak.");
            if (left.Y != right.Y)
                throw new BadParameterException("A bal és jobb pontok nem azonos magasságban vannak.");

            _Center = new Point((left.X + right.X) / 2, left.Y);
            _Radious = Math.Abs(left.X - right.X) / 2;
        }

        /// <summary>
        /// Megtalálható-e az alakzatban egy adott pont.
        /// </summary>
        /// <param name="point">A keresendő pont.</param>
        /// <returns>Igaz, ha a pont az alakzaton belül van.</returns>
        public override Boolean Contains(Point point)
        {
            return (_Center.Y == point.Y && Math.Abs(point.X - _Center.X) < _Radious);                
        }
    }

    /// <summary>
    /// Függőleges vonal típusa.
    /// </summary>
    class VerticalLine : Shape
    {
        /// <summary>
        /// Alakzat kerületének lekérdezése.
        /// </summary>
        public override Double Perimeter
        {
            get { return 2 * _Radious; }
        }

        /// <summary>
        /// Függőleges vonal létrehozása.
        /// </summary>
        /// <param name="center">Középpont.</param>
        /// <param name="length">Hossz.</param>
        public VerticalLine(Point center, Double length)
            : base(center, length / 2)
        { }

        /// <summary>
        /// Függőleges vonal létrehozása.
        /// </summary>
        /// <param name="left">Bal végpont.</param>
        /// <param name="right">Jobb végpont.</param>
        public VerticalLine(Point left, Point right)
        {
            if (left.X == right.X && left.Y == right.Y)
                throw new BadParameterException("A bal és jobb pontok azonosak.");
            if (left.X != right.X)
                throw new BadParameterException("A bal és jobb pontok nem azonos szélességben vannak.");

            _Center = new Point(left.X, (left.Y + right.Y) / 2);
            _Radious = Math.Abs(left.Y - right.Y) / 2;
        }

        /// <summary>
        /// Megtalálható-e az alakzatban egy adott pont.
        /// </summary>
        /// <param name="point">A keresendő pont.</param>
        /// <returns>Igaz, ha a pont az alakzaton belül van.</returns>
        public override Boolean Contains(Point point)
        {
            return (_Center.X == point.X && Math.Abs(point.Y - _Center.Y) < _Radious);
        }
    }
}
