﻿using System;
using System.Collections.Generic;
using System.Text;

namespace ELTE.Basics.GeometricShapes
{
    class Program
    {       
        static void Main(string[] args)
        {
            List<Shape> shapesList = new List<Shape>();
            Fill(shapesList);

            Console.WriteLine("A teljes terület: " + Territory(shapesList));
            Console.WriteLine("A teljes kerület: " + Perimeter(shapesList));

            Double x, y;
            try
            {
                Console.Write("Keresett pont X: ");
                x = Convert.ToInt32(Console.ReadLine());
                Console.Write("Keresett pont Y: ");
                y = Convert.ToInt32(Console.ReadLine());
                Console.WriteLine("A keresett pont " + FindPoint(shapesList, new Point(x, y)) + " alakzatban található meg.");
            }
            catch (FormatException)
            {
                Console.WriteLine("Hibás bemenet, a pontkeresés nem végezhető el.");
            }
            Console.ReadKey();
        }

        static void Fill(List<Shape> shapes)
        {
            Double x, y, z;
            
            Int32 selection = GetChoice();
            while (selection != 0)
            {
                try
                {
                    Console.Write("Középpont X: ");
                    x = Convert.ToDouble(Console.ReadLine());
                    Console.Write("Középpont Y: ");
                    y = Convert.ToDouble(Console.ReadLine());
                    switch (selection)
                    {
                        case 1:
                            Console.Write("Hossz: ");
                            z = Convert.ToDouble(Console.ReadLine());
                            shapes.Add(new HorisontalLine(new Point(x, y), z));
                            break;
                        case 2:
                            Console.Write("Hossz: ");
                            z = Convert.ToDouble(Console.ReadLine());
                            shapes.Add(new VerticalLine(new Point(x, y), z));
                            break;
                        case 3:
                            Console.Write("Sugár: ");
                            z = Convert.ToDouble(Console.ReadLine());
                            shapes.Add(new Circle(new Point(x, y), z));
                            break;
                        case 4:
                            Console.Write("Oldalhossz: ");
                            z = Convert.ToDouble(Console.ReadLine());
                            shapes.Add(new Rectangle(new Point(x, y), z));
                            break;
                    }
                }
                catch (FormatException ex)
                {
                    Console.WriteLine("Hibás bemenet: " + ex.Message);
                }
                catch (BadParameterException ex)
                {
                    Console.WriteLine("Hibás paraméter: " + ex.Message);
                }
                finally
                {
                    selection = GetChoice();
                }

            }

        }
        static Int32 GetChoice()
        {
            Console.WriteLine("Következő alakzat:");
            Console.WriteLine("1 - vízszintes vonal");
            Console.WriteLine("2 - függőleges vonal");
            Console.WriteLine("3 - kör");
            Console.WriteLine("4 - négyzet");
            Console.WriteLine("0 - Bemenet vége");

            Int32 selection = 0;
            Boolean loop = false;
            do
            {
                try
                {
                    selection = Convert.ToInt32(Console.ReadLine());
                    if (selection < 0 || selection > 4)
                    {
                        Console.WriteLine("0 és 4 közötti számot kérek!");
                        loop = true;
                    }
                }
                catch (FormatException)
                {
                    Console.WriteLine("0 és 4 közötti számot kérek!");
                    loop = true;
                }
            } while (loop);

            return selection;
        }
        static Double Perimeter(List<Shape> shapes)
        {
            Double sum = 0;
            for (Int32 i = 0; i < shapes.Count; i++)
                sum += shapes[i].Perimeter;
            return sum;
        }
        static Double Territory(List<Shape> shapes)
        {
            Double sum = 0;
            for (Int32 i = 0; i < shapes.Count; i++)
                sum += shapes[i].Territory;
            return sum;
        }
        static Int32 FindPoint(List<Shape> shapes, Point point)
        {
            Int32 count = 0;
            for (Int32 i = 0; i < shapes.Count; i++)
                if (shapes[i].Contains(point))
                    count++;
            return count;
        }
    }
}
