﻿using System;
using System.Collections.Generic;
using System.Text;

namespace ELTE.Basics.GeometricShapes
{
    /// <summary>
    /// Absztrakt geometriai alakzat osztály
    /// </summary>
    abstract class Shape
    {
        protected Point _Center;
        protected Double _Radious;

        /// <summary>
        /// Alakzat területének lekérdezése.
        /// </summary>
        public virtual Double Territory 
            // virtuális mező, tehát felüldefiniálható a leszármazottban
        { 
            get { return 0; } 
        }
        /// <summary>
        /// Alakzat kerületének lekérdezése.
        /// </summary>
        public abstract Double Perimeter
            // absztrakt mező, tehát a leszármazottban felül kell definiálni
        { 
            get; // csak azt adjuk meg, hogy lekérdezés lesz, a törzsét nem
        }
        /// <summary>
        /// Középpont lekérdezése
        /// </summary>
        public Point Center { get { return _Center; } }

        /// <summary>
        /// Alakzat létrehozása.
        /// </summary>
        public Shape()
        {
            _Center = new Point(0, 0);
            _Radious = 0;
        }

        /// <summary>
        /// Alakzat létrehozása.
        /// </summary>
        /// <param name="center">Középpont.</param>
        /// <param name="radious">Sugár.</param>
        public Shape(Point center, Double radious)
        {
            if (radious < 0)
                throw new BadParameterException("A sugár nem lehet negatív szám.");
            _Center = center; _Radious = radious;
        }

        /// <summary>
        /// Alakzat eltolása.
        /// </summary>
        /// <param name="x">Vízszintes érték.</param>
        /// <param name="y">Függőleges érték.</param>
        public void Move(Double x, Double y)
        {
            _Center.Move(x, y);
        }
        /// <summary>
        /// Alakzat eltolása.
        /// </summary>
        /// <param name="v">Eltolás vektora.</param>
        public void Move(Vector v)
        {
            _Center.Move(v);
        }
        /// <summary>
        /// Megtalálható-e az alakzatban egy adott pont.
        /// </summary>
        /// <param name="point">A keresendő pont.</param>
        /// <returns>Igaz, ha a pont az alakzaton belül van.</returns>
        public abstract Boolean Contains(Point point);
            // absztrakt metódus, nincs törzse, leszármazottban felül kell definiálni
    }
}
